/*
 * Name: RL_ImageFrame.cpp
 * Desc: Main UI for the ImageFrame Widget.
 *		Used for displaying the frame sequences.
 *
 * Issue #: 585
 *
 * Author: Jihyun Cho
 *
 * Revision History:
 * -----------------
 *	03/01/2010 ---> Initial Implementation
 *	05/10/2010 ---> Fix for reducing jitter on displaying frames
 */

#include "RL_ImageFrame.hpp"

// ImageFrame Constructor.
// Parameters are needed to use Glade

/*! \brief The constructor called from glade. This constructor is needed for the derived widgets
	@param BaseObjectType* cobject - used by glade
	@param Glib::RefPtr<Gtk::Builder>& refGlade - used by glade
 */
RL_ImageFrame::RL_ImageFrame(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade) : Gtk::DrawingArea(cobject)
{
	imageWidth = 0;
	imageHeight = 0;
	steps = 0;

	isInitialized = false;

	// connect the signals
	Glib::signal_timeout().connect(sigc::mem_fun(*this, &RL_ImageFrame::Timeout), 10 );
	signal_expose_event().connect(sigc::mem_fun(*this, &RL_ImageFrame::ExposeEvent), false);
}

// ImageFrame Default Constructor.
/*! \brief The constructor for RL_ImageFrame */
RL_ImageFrame::RL_ImageFrame()
{
	imageWidth = 0;
	imageHeight = 0;
	steps = 0;

	isInitialized = false;

	// connect the signals
	Glib::signal_timeout().connect(sigc::mem_fun(*this, &RL_ImageFrame::Timeout), 10 );
	signal_expose_event().connect(sigc::mem_fun(*this, &RL_ImageFrame::ExposeEvent), false);
}

// Deconstructor
/*! \brief The deconstructor for RL_ImageFrame */
RL_ImageFrame::~RL_ImageFrame() 
{
	if(isInitialized) {
		delete buf;
	}
}

// Initialization
// ImageFrame must be initialized before use
/*! \brief Initialization procedure for RL_ImageFrame. Frames must be initialized and may be initialized only once.

	@param unsigned int width	- width
	@param unsigned int height	- height
 */
void RL_ImageFrame::Initialize(unsigned int width, unsigned int height) {
	// check if the widget is already initialized
	if(!isInitialized) {
		// initialize
		isInitialized = true;

		imageWidth = width;
		imageHeight = height;

		steps = width*3;

		buf = new unsigned char[width*height*3];
	}
}

/*! \brief Rotate the frame 180 degrees

	@param bool set - true if rotate the frame 180 degree, otherwise false.
 */
void RL_ImageFrame::SetFlip(bool set)
{
	Glib::Mutex::Lock saveLock(imageMutex);
	isFlip = set;
}

//Change the buffer to a new frame
/*! \brief Update the image buffer for the display 

	@param unsigned char* data	- new image frame

	@return bool	- returns false if the widget is not initialized
 */
bool RL_ImageFrame::UpdateBuffer(unsigned char* data) {
	Glib::Mutex::Lock saveLock(imageMutex);

	// stop if the RL_ImageFrame is not initialized
	if(!isInitialized) {
		printf("NOT INITIALIZED!\n");
		return false;
	}

	// get correct buffer size
	int imageSize = imageWidth*imageHeight;
	int j = 0;

	if(isFlip) {
		for(int i = imageSize ; i > 0 ; i--) {
			buf[i*3-1] = data[j+2];
			buf[i*3-2] = data[j+1];
			buf[i*3-3] = data[j];
			j+=3;
		}
		// copy the data to buffer
		pixbuf = Gdk::Pixbuf::create_from_data(buf, Gdk::COLORSPACE_RGB, false, 8, imageWidth, imageHeight, steps);
	}
	else
		pixbuf = Gdk::Pixbuf::create_from_data(data, Gdk::COLORSPACE_RGB, false, 8, imageWidth, imageHeight, steps);


	Glib::RefPtr<Gdk::Pixbuf> temp = pixbuf->copy();

	buf_list.push_back(temp);

//	buf_list.clear();
//	buf_list.push_back(temp);
	while(buf_list.size() >= 5)
		buf_list.pop_front();
	//	queue_draw();

	return true;
}

// Redraw window when it is necessary
/*! \brief Called by main Gtk Thread 
	@param GdkEventExpose* e	-	Expose event from GDK
	@return true
 */
bool RL_ImageFrame::ExposeEvent(GdkEventExpose* e){
	Glib::Mutex::Lock saveLock(imageMutex);
	// get current window widget
	Glib::RefPtr<Gdk::Window> window = get_window();

	// if buffer is not empty, window is not null, and RL_ImageFrame is initialized
	if((buf_list.size() > 0) && window && isInitialized) {
		// create Pixbuf
		// draw the Pixbuf
		Glib::RefPtr<Gdk::Pixbuf> list_pixbuf = buf_list.front();
		list_pixbuf->render_to_drawable(get_window(), get_style()->get_black_gc(), 0, 0, 0, 0, pixbuf->get_width(), pixbuf->get_height(),
				Gdk::RGB_DITHER_NONE, 0, 0);

		if(buf_list.size() > 1) {
			buf_list.pop_front();
		}
	}
	return true;
}

// Update the frame on timeout
/*! \brief Forces the image widget to redraw itself
	@return true
 */
bool RL_ImageFrame::Timeout() {
	Glib::Mutex::Lock saveLock(imageMutex);
	Glib::RefPtr<Gdk::Window> window = get_window();

	if (window && isInitialized)
	{
		// force the image frame area to redraw
		Gdk::Rectangle r(0, 0, get_allocation().get_width(), get_allocation().get_height());
		window->invalidate_rect(r, false);
	}
	return true;
}
