/* Name: IntrinsicParams.cpp
*  Desc: Class for handling intrinsic parameters for the stereo
*
*  Authors: Taha Kowsari, Jihyun Cho
*
*  Revision History:
*  -----------------
*    06/18/2010 --> Initial Implementation by Taha Kowsari.
*    12/20/2010 --> path selection added.
*     
*/
#include "RL_IntrinsicParams.h"

/*! \brief Constructor: Initializes the Intrinsic parameter object.
 */
RL_IntrinsicParams::RL_IntrinsicParams() {
	CameraMatrix = cvMat(3, 3, CV_64F, _camera );
	Distortion = cvMat(1, 5, CV_64F, _dist_coeffs);
}

RL_IntrinsicParams::~RL_IntrinsicParams() {
}

/*! \brief WriteToFile: Writes the intrinsics to "intr_<serialnumber>.xml" file 
	@param unsigned id - serial number of the camera.
	@param const char* path - path of where the intrinsic xml file is going to be.
 */
void RL_IntrinsicParams::WriteToFile(unsigned id, const char* path)
{
	char filename[200];
	sprintf(filename, "%s/intr_%d.xml", path, id);
    CvFileStorage* storage = cvOpenFileStorage( filename, 0, CV_STORAGE_WRITE_TEXT);
    cvWrite( storage, "CameraMatrix", &CameraMatrix);
    cvWrite( storage, "Distortion", &Distortion);
    cvWriteInt( storage, "Size_Width", imageSize.width);
    cvWriteInt( storage, "Size_Height", imageSize.height);
    cvReleaseFileStorage( &storage );
}

/*! \brief ReadFromFile: Reads the intrinsics from "intr_<serialnumber>.xml" file
	@param unsigned id - serial number of the camera.
	@param const char* path - path of intrinsic xml file.
	@return bool returns true on success, false otherwise.
 */
bool RL_IntrinsicParams::ReadFromFile(unsigned id, const char* path)
{
	char filename[200];
	sprintf(filename, "%s/intr_%d.xml", path, id);
    CvFileStorage* storage = cvOpenFileStorage( filename, 0, CV_STORAGE_READ);
    if(storage == NULL)
    	return false;
    CvMat* cameraMatrix = (CvMat*)cvReadByName(storage,NULL, "CameraMatrix");
    cvCopy(cameraMatrix, &CameraMatrix);

    CvMat* distortion = (CvMat*)cvReadByName(storage,NULL, "Distortion");
    cvCopy(distortion, &Distortion);

    imageSize.width = cvReadIntByName(storage,NULL, "Size_Width");
    imageSize.height = cvReadIntByName(storage,NULL, "Size_Height");

    cvReleaseFileStorage(&storage);

    cvReleaseMat(&cameraMatrix);
    cvReleaseMat(&distortion);

    return true;
}
