/*
* Name: RL_SequenceFileHeader.cpp
* Desc: The file header structure for the stereo image sequence
*
* Issue #: 585
*
* Author: Jihyun Cho
*
* Revision History:
* -----------------
*	07/01/2010 ---> Initial Implementation
*	12/20/2010 ---> Restructured
*/

#include "RL_SequenceFileHeader.hpp"

/*! \brief RL_SequenceFileHeader: Constructor for writing the header
	
	@param unsigned int nCam	- number of cameras
	@param unsigned int* serials	- serial numbers for the cameras
*/
RL_SequenceFileHeader::RL_SequenceFileHeader(unsigned int nCam, unsigned int* serials) {
	char temp[23] = VERSION_0_TYPE;
	memcpy(seqFileHeader.filetype, temp, sizeof(char)*23);
	seqFileHeader.numCameras = nCam;

	for(unsigned int i = 0; i < nCam; i++) {
		seqFileHeader.serialNumbers[i] = serials[i];
	}

	// set default values
	seqFileHeader.format = INVALID_COLORFORMAT;
	seqFileHeader.fps = INVALID_FPS;
	seqFileHeader.width = 0;
	seqFileHeader.height = 0;
}

/*! \brief RL_SequenceFileHeader: Default constructor
*/
RL_SequenceFileHeader::RL_SequenceFileHeader() {
	// set default values
	char temp[23] = VERSION_0_TYPE;
	memcpy(seqFileHeader.filetype, temp, sizeof(char)*23);

	seqFileHeader.numCameras = 0;

	for(int i = 0; i < MAX_NUM_CAMERAS; i++) 
		seqFileHeader.serialNumbers[i] = 0;

	seqFileHeader.format = INVALID_COLORFORMAT;
	seqFileHeader.fps = INVALID_FPS;
	seqFileHeader.width = 0;
	seqFileHeader.height = 0;
}


RL_SequenceFileHeader::~RL_SequenceFileHeader() {
}

/*! \brief SetResolution: Set frame size
	
	@param int width	- the width of the frame
	@param int height	- the height of the frame
*/
void RL_SequenceFileHeader::SetResolution(int width, int height) {
	seqFileHeader.width = width;
	seqFileHeader.height = height;
}

/*! \brief SetCameras: Sets serial numbers of the cameras
	
	@param unsigned int* serials	- camera serial numbers
	@param unsigned int numOfCameras	- number of cameras
*/
void RL_SequenceFileHeader::SetCameras(unsigned int* serials, unsigned int numOfCameras) {
	for(unsigned int i = 0; i < numOfCameras; i++) {
		seqFileHeader.serialNumbers[i] = serials[i];
	}
}

/*! \brief SetFPS: Sets FPS
	
	@param FPS fps	- Frames per second
*/
void RL_SequenceFileHeader::SetFPS(FPS fps) {
	seqFileHeader.fps = fps;
}

/*! \brief SetColor: Sets color format
	
	@param COLORFORMAT color	- color format
*/
void RL_SequenceFileHeader::SetColor(COLORFORMAT color) {
	seqFileHeader.format = color;
}

/*! \brief GetNumberOfCameras: returns number of cameras
	
	@return unsigned int	- number of cameras
*/
unsigned int RL_SequenceFileHeader::GetNumberOfCameras() {
	return seqFileHeader.numCameras;
}

/*! \brief GetWidth: returns frame width
	
	@return unsigned int	- frame width
*/
unsigned int RL_SequenceFileHeader::GetWidth() {
	return seqFileHeader.width;
}

/*! \brief GetHeight: returns frame height
	
	@return unsigned int	- frame height
*/
unsigned int RL_SequenceFileHeader::GetHeight() {
	return seqFileHeader.height;
}

/*! \brief GetSerials: stores serial numbers to a given pointer
	
	@param unsigned int* serials	- pointer to store the serial numbers
*/
void RL_SequenceFileHeader::GetSerials(unsigned int* serials) {
	for(int i = 0; i < seqFileHeader.numCameras; i++)
		serials[i] = seqFileHeader.serialNumbers[i];
}

/*! \brief GetVersion: returns file version
	
	@return VERSION	- file version
*/
VERSION RL_SequenceFileHeader::GetVersion() {
	if(strcmp(seqFileHeader.filetype, VERSION_0_TYPE) == 0) 
		return VERSION_0;

	return INVALID_VERSION;
}

/*! \brief GetFPS: returns FPS
	
	@return FPS	- FPS
*/
FPS RL_SequenceFileHeader::GetFPS() {
	return seqFileHeader.fps;
}

/*! \brief GetHeader: returns file header structure
	
	@return SEQUENCE_HEADER*	- file header structure
*/
SEQUENCE_HEADER* RL_SequenceFileHeader::GetHeader() {
	return &seqFileHeader;
}

/*! \brief GetColor: returns color format
	
	@return COLORFORMAT	- color format
*/
COLORFORMAT RL_SequenceFileHeader::GetColor() {
	return seqFileHeader.format;
}
