/*
* Name: RL_SequenceReader.cpp
* Desc: RL_SequenceReader class is used for reading the stereo image sequence.
*
* Issue #: 585
*
* Author: Jihyun Cho
*
* Revision History:
* -----------------
*	03/01/2010 ---> Initial Implementation
*/

#include "RL_SequenceReader.hpp"

/*! \brief Default constructor for RL_SequenceReader. Initiates new RL_GPSData and RL_CANData */
RL_SequenceReader::RL_SequenceReader() 
{
	pFile = NULL;
	frameImage = NULL;
	gpsData = new RL_GPSData();
	canData = new RL_CANData();
}

/*! \brief Constructor for RL_SequenceReader with the file name. 
Initiates new RL_GPSData and RL_CANData, then opens a sequence file. */
RL_SequenceReader::RL_SequenceReader(char* filename) 
{
	pFile = NULL;
	frameImage = NULL;
	gpsData = new RL_GPSData();
	canData = new RL_CANData();
	OpenFile(filename);
}

/*! \brief Deconstructor */
RL_SequenceReader::~RL_SequenceReader() 
{
	if(frameImage != NULL) {
		for(unsigned int i = 0; i < numberOfCam; i++) {
			cvReleaseImage(&frameImage[i]);
		}

		delete [] serials;
		delete [] lastCycle;
		delete [] lastCount;
		delete [] frameImage;
	}

	delete gpsData;
	delete canData;
}

/*! \brief  SetFile: opens new file. If there is a file already open, the function will assume the sequence was recorded with the same configuration as previously opened sequences.
	@param char *filename	- File name

	@return bool	- 	true if the input stream opens successfully
						false if opening the input stream fails
*/
bool RL_SequenceReader::SetFile(char* filename) {
	if(pFile != NULL)
		CloseFile();

	return OpenFile(filename);
}


/*! \brief  OpenFile: opens file, and returns file header
	@param char *filename	- File name

	@return bool	- 	true if the input stream opens successfully
						false if opening the input stream fails
*/
bool RL_SequenceReader::OpenFile(char* filename) 
{
	pFile = fopen(filename, "rb");
	if(pFile != NULL) {
		// Read Sequence File Header
		if(fread(fileHeader.GetHeader(), 1, sizeof(SEQUENCE_HEADER), pFile) != sizeof(SEQUENCE_HEADER)) {
			fclose(pFile);
			pFile = NULL;
			fprintf(stderr, "Failed to open file %s\n", filename);

			return false;
		}

		//Set Size of data
		dataSize = fileHeader.GetWidth()*fileHeader.GetHeight()*(fileHeader.GetColor() == RGB ? 3 : 1);
		canbusSize = sizeof(CANStruct)*HIGH_CANBUS_LENGTH;
		gpsDataSize = sizeof(gps_data);

		//Allocate memory for frames
		numberOfCam = fileHeader.GetNumberOfCameras();

		if(frameImage == NULL ||
			frameImage[0]->height != fileHeader.GetHeight() ||
			frameImage[0]->width != fileHeader.GetWidth() ||
			frameImage[0]->nChannels != (fileHeader.GetColor() == RGB ? 3 : 1)) {

			if(frameImage != NULL)
				for(unsigned int i = 0; i < numberOfCam; i++) {
					cvReleaseImage(&frameImage[i]);
				}
			else {
				lastCycle = new int[numberOfCam];
				lastCount = new int[numberOfCam];
				serials = new unsigned int[numberOfCam];

				fileHeader.GetSerials(serials);
				frameImage = new IplImage*[numberOfCam];
			}

			for(unsigned int i = 0; i < numberOfCam; i++) {
				frameImage[i] = cvCreateImage(cvSize(fileHeader.GetWidth(), fileHeader.GetHeight()), IPL_DEPTH_8U, fileHeader.GetColor() == RGB ? 3 : 1);
			}

		}

		if(fileHeader.GetVersion() == INVALID_VERSION) {
			//Incorrect filetype
			fprintf(stderr, "Incorrect filetype %s\n", filename);
			return false;
		}

		return true;
	}
	
	fprintf(stderr, "Failed to open file %s\n", filename);
	return false;
}

/*! \brief  ReadFrame: read single frame from input stream and store data 	
	@return bool	- 	true if read input stream successfully
						false if reading input stream fails			
*/
bool RL_SequenceReader::ReadFrame()
{
	if(!pFile || feof(pFile))	return false;

	// Read CANBus Info
	if(fread(canData->getCANStruct(), 1, canbusSize, pFile) != (size_t)canbusSize) return false;

	// Read GPS Info
	if(fread(gpsData->GetData(), 1, gpsDataSize, pFile) != sizeof(gps_data)) return false;

	// Read Frames
	for(unsigned int i = 0; i < numberOfCam; i++) {
		if(fread(&lastCycle[i], 1, sizeof(int), pFile) != sizeof(int)) return false;
		if(fread(&lastCount[i], 1, sizeof(int), pFile) != sizeof(int)) return false;
		if(fread(frameImage[i]->imageData, 1, dataSize, pFile) != (size_t)dataSize) return false;
	}

	return true;
}

/*! \brief  GrabFrame: Returns last image frame read.
	@param int i	- image frame index
	@return IplImage*	- 	returns NULL in case of error
*/
IplImage* RL_SequenceReader::GrabFrame(int i) {
	if( i < 0 || i >= (int)numberOfCam || pFile == NULL) 
		return NULL;

	return frameImage[i];
}

/*! \brief  GetGPSData: Returns last GPS data.

	@return RL_GPSData*	- 	The last GPS data read
*/
RL_GPSData* RL_SequenceReader::GetGPSData() {
	return gpsData;
}

/*! \brief  GrabSerials: Returns the serial number for the image frame with given index. This function is required in order to load correct calibration parameters.
	@param int i	- image frame index

	@return unsigned int 	- 	a serial number of the camera
*/
unsigned int RL_SequenceReader::GrabSerials(int i) {
	if( i < 0 || i >= (int)numberOfCam || pFile == NULL) return 0;
	
	return serials[i];
}

/*! \brief  GetNumberOfFrames: Returns the number of cameras used to record the sequence

	@return unsigned int 	- 	number of cameras used to record the sequence
*/
unsigned int RL_SequenceReader::GetNumberOfFrames() {
	return fileHeader.GetNumberOfCameras();
}

/*! \brief  GetFrameWidth: Returns the width of each image frame

	@return int 	- 	The width of the image frame
*/
int RL_SequenceReader::GetFrameWidth() {
	return fileHeader.GetWidth();
}

/*! \brief  GetFrameHeight: Returns the height of each image frame

	@return int 	- 	The height of the image frame
*/
int RL_SequenceReader::GetFrameHeight() {
	return fileHeader.GetHeight();
}

/*! \brief  GetNumberOfChannels: Returns number of channels in recorded image frame

	@return int 	- 	1 for grayscale and 3 for RGB image
*/
int RL_SequenceReader::GetNumberOfChannels() {
	return fileHeader.GetColor() == RGB ? 3 : 1;
}


/*! \brief Close input stream */
bool RL_SequenceReader::CloseFile() 
{
	if(pFile != NULL) {
		fclose(pFile);

		pFile = NULL;
		return true;
	}

	return false;
}

