/* Name: RL_ViewGUI.cpp
*  Desc: RL_Reader GUI
*
*  Authors: Jihyun Cho, Taha Kowsari
*
*  Revision History:
*  -----------------
*    12/20/2010 --> Initial Implementation
*     
*/

#include "RL_ViewGUI.hpp"

/*! \brief RL_ViewGUI(): default Constructor
*/
RL_ViewGUI::RL_ViewGUI()
{
	filenameBuf = new char[255];
	inputStream = new RL_SequenceReader();

	flipStatus = false;
	serials = NULL;

	grabLoop = NULL;
	SetThreadStatus( false );

	stereoProcessor = NULL;

	// Get main window
	refBuilder = Gnome::Glade::Xml::create("resource/rl_calib.glade");

	// Assign widgets from glade

	refBuilder->get_widget("folderChooser", folderChooser);
	refBuilder->get_widget("calibWindow", window);
	refBuilder->get_widget_derived("imageFrame1", imageFrame[0]);
	refBuilder->get_widget_derived("imageFrame2", imageFrame[1]);
	refBuilder->get_widget_derived("rectified1", rectified[0]);
	refBuilder->get_widget_derived("rectified2", rectified[1]);
	refBuilder->get_widget_derived("stereoFrame", stereoFrame);

	// Spin Buttons
	refBuilder->get_widget("uniqueSpin", uniqueSpin);
	refBuilder->get_widget("thresholdSpin", thresholdSpin);
	refBuilder->get_widget("filterSizeSpin", filterSizeSpin);
	refBuilder->get_widget("filterCapSpin", filterCapSpin);
	refBuilder->get_widget("windowSizeSpin", windowSizeSpin);
	refBuilder->get_widget("speckleSpin", speckleSpin);
	refBuilder->get_widget("speckleWindowSpin", speckleWindowSpin);
	refBuilder->get_widget("disparityScale", disparityScale);

	// Play Buttons
	refBuilder->get_widget("playButton", playButton);
	refBuilder->get_widget("pauseButton", pauseButton);
	refBuilder->get_widget("stopButton", stopButton);

	// Frame Selectors
	refBuilder->get_widget("frameSelector1", frameSelector[0]);
	refBuilder->get_widget("frameSelector2", frameSelector[1]);

	// Default Values
	imageFrame[0]->Initialize(320, 240);
	imageFrame[0]->SetFlip(false);
	imageFrame[1]->Initialize(320, 240);
	imageFrame[1]->SetFlip(false);
	rectified[0]->Initialize(320, 240);
	rectified[0]->SetFlip(false);
	rectified[1]->Initialize(320, 240);
	rectified[1]->SetFlip(false);
	stereoFrame->Initialize(320, 240);
	stereoFrame->SetFlip(false);
	frameSelector[0]->set_sensitive(false);
	frameSelector[1]->set_sensitive(false);
	folderPath = folderChooser->get_current_folder();
	camOnFrame[0] = -1;
	camOnFrame[1] = -1;

	OnStop();
	playButton->set_sensitive(OpenFile(0));

	// assign event callbacks
	if(playButton) {
		playButton->signal_clicked().connect(sigc::mem_fun(*this, &RL_ViewGUI::OnPlay));
	}
	if(pauseButton) {
		pauseButton->signal_clicked().connect(sigc::mem_fun(*this, &RL_ViewGUI::OnPause));
	}

	if(stopButton) {
		stopButton->signal_clicked().connect(sigc::mem_fun(*this, &RL_ViewGUI::OnStop));
	}

	if(folderChooser) {
		folderChooser->signal_current_folder_changed().connect(sigc::mem_fun(*this, &RL_ViewGUI::SelectFolder));
	}

	if(frameSelector[0]) {
		frameSelector[0]->signal_changed().connect(sigc::bind<int>(sigc::mem_fun(*this, &RL_ViewGUI::SelectFrame), 0));
	}

	if(frameSelector[1]) {
		frameSelector[1]->signal_changed().connect(sigc::bind<int>(sigc::mem_fun(*this, &RL_ViewGUI::SelectFrame), 1));
	}

	stopEvent.connect(sigc::mem_fun(*this, &RL_ViewGUI::OnStop));

	window->signal_delete_event().connect(sigc::mem_fun( *this, &RL_ViewGUI::OnDestroy ));
	window->set_title("RoadLab - Demo Application");

	// Set logo
	Glib::RefPtr< Gdk::Pixbuf > buf1 = Gdk::Pixbuf::create_from_file(LOGO_IMAGE);
	memcpy(logobuf, buf1->get_pixels(), 320*240*3);

	SetLogo(rectified[0], logobuf);
	SetLogo(rectified[1], logobuf);
	SetLogo(stereoFrame, logobuf);
	SetLogo(imageFrame[0], logobuf);
	SetLogo(imageFrame[1], logobuf);

	window->show();
}

/*! \brief ~RL_ViewGUI(): destructor
*/
RL_ViewGUI::~RL_ViewGUI() {
	if(NULL != grabLoop) {
		SetThreadStatus( false );
		grabLoop->join();
	}

	if(serials != NULL)
		delete [] serials;

	if(stereoProcessor != NULL)
		delete stereoProcessor;

	delete [] filenameBuf;
	delete inputStream;
}

/*! \brief SetLogo: setting the logo on a given RL_ImageFrame with a buffered image

	@param RL_ImageFrame* Frame	- a pointer to RL_ImageFrame widget for setting the logo
	@param guint8* log_buf	- a buffered logo
*/
void RL_ViewGUI::SetLogo(RL_ImageFrame* Frame, guint8* log_buf) {
	Frame->UpdateBuffer(log_buf);
}

/*! \brief DisparityCheck: used to fix the value of disparity values from the ScaleH widget for the disparity
*/
int RL_ViewGUI::DisparityCheck() {
	int value = (int)disparityScale->get_value();
	int remainder = value % 16;

	value = remainder > 8 ? value + (16 - remainder):value - remainder;
	return value;
}

/*! \brief UpdateStereoParam: updates stereo parameters for selected frames
*/
void RL_ViewGUI::UpdateStereoParam() {
	// Update Stereo Param
	Glib::Mutex::Lock stereoLock(stereoMutex);
	if(stereoProcessor != NULL && stereoProcessor->parametersSet) {
		int disp = DisparityCheck();
		disparityScale->set_value((double)disp);

		stereoParams.disparity = disp;
		stereoParams.filterCap = filterCapSpin->get_value_as_int();
		stereoParams.filterSize = filterSizeSpin->get_value_as_int();
		stereoParams.speckleRange = speckleSpin->get_value_as_int();
		stereoParams.speckleWindowSize = speckleWindowSpin->get_value_as_int();
		stereoParams.windowSize = windowSizeSpin->get_value_as_int();
		stereoParams.unique = uniqueSpin->get_value_as_int();
		stereoParams.thresh = thresholdSpin->get_value_as_int();

		stereoProcessor->SetStereoParams(stereoParams);
	}
}

/*! \brief SelectFolder: a callback function for the folder selection
*/
void RL_ViewGUI::SelectFolder() {
	folderPath = folderChooser->get_current_folder();

	if(OpenFile(0)) {
		playButton->set_sensitive(true);

		if(serials != NULL)
			delete [] serials;

		if(stereoProcessor != NULL)
			delete stereoProcessor;

		serials = new unsigned int[inputStream->GetNumberOfFrames()];
		stereoProcessor = new RL_StereoProcessor(inputStream->GetFrameWidth(), inputStream->GetFrameHeight());

		// get serial numbers for the recorded frames
		for(unsigned int i = 0; i < inputStream->GetNumberOfFrames(); i++)
			serials[i] = inputStream->GrabSerials(i);

		// Build list of available frames
		ModelColumns mColumns;
		Glib::RefPtr<Gtk::ListStore> mRefTreeModel = Gtk::ListStore::create(mColumns);

		// First Row
		Gtk::TreeModel::Row row = *(mRefTreeModel->append());
		row[mColumns.mColName] = "Not Selected";

		for(unsigned int i = 0; i < inputStream->GetNumberOfFrames();i++) {
			// Set rows
			char buffer [50];
			sprintf (buffer, "Frame# %d", i);
			Gtk::TreeModel::Row row = *(mRefTreeModel->append());
			row[mColumns.mColName] = buffer;
		}

		frameSelector[0]->clear();
		frameSelector[0]->set_model(mRefTreeModel);
		frameSelector[0]->pack_start(mColumns.mColName);
		frameSelector[0]->set_active(0);
		frameSelector[0]->set_sensitive(true);

		frameSelector[1]->clear();
		frameSelector[1]->set_model(mRefTreeModel);
		frameSelector[1]->pack_start(mColumns.mColName);
		frameSelector[1]->set_active(0);
		frameSelector[1]->set_sensitive(true);
		
		fileOpened = true;
	}
	else {
		playButton->set_sensitive(false);
	}
}

/*! \brief SelectFrame: a callback function for the frame selection

	@param int i	- RL_ImageFrame widget index
*/
void RL_ViewGUI::SelectFrame(int i) {
	int selection = frameSelector[i]->get_active_row_number();

	SetFrameIndex(i, selection);
}

/*! \brief SetFrameIndex: update frame selections

	@param int i - RL_ImageFrame widget index
	@param int n - selection
*/
void RL_ViewGUI::SetFrameIndex(int i, int n) {
	Glib::Mutex::Lock saveLock(frameMutex);
	camOnFrame[i] = n - 1;
}


/*! \brief GetFrameIndex: get frame selections

	@param int i	- RL_ImageFrame widget index

	@return int returns selected frame index for the RL_ImageFRame widget
*/
int RL_ViewGUI::GetFrameIndex(int i) {
	Glib::Mutex::Lock saveLock(frameMutex);
	return camOnFrame[i];
}

/*! \brief SetThreadStatus: set the thread status for frame grabbing

	@param bool runStatus	- set false to stop the frame grabbing thread
*/
void RL_ViewGUI::SetThreadStatus(bool runStatus)
{
	Glib::Mutex::Lock saveLock(runMutex);
	threadRun = runStatus;
}

/*! \brief GetThreadStatus: get the thread status for frame grabbing
	@return bool returns true if the frame grabbing thread is running
*/
bool RL_ViewGUI::GetThreadStatus()
{
	Glib::Mutex::Lock saveLock(runMutex);
	return threadRun;
}

/*! \brief UpdateFrames: starts frame grabbing thread
*/
void RL_ViewGUI::UpdateFrames() {
	grabLoop = Glib::Thread::create( sigc::mem_fun(*this, &RL_ViewGUI::GrabFrames), true );
}

/*! \brief OpenFile: opens nth RoadLab sequence
	
	@param int n	- sequence file index

	@return bool	- returns true if the file is opened successfully
*/
bool RL_ViewGUI::OpenFile(int n) {
	filenumber = n;
	sprintf(filenameBuf, "%s/%010d.rds", folderPath.c_str(), filenumber);

	return inputStream->SetFile(filenameBuf);
}

/*! \brief OnPlay: a callback function for the Play button	
*/
void RL_ViewGUI::OnPlay() {
	Glib::Mutex::Lock saveLock(playMutex);

	if(!pausing) {
		if(GetThreadStatus()) {
			SetThreadStatus( false );
			grabLoop->join();
			grabLoop = NULL;
		}

		if(!fileOpened) OpenFile(0);

		SetThreadStatus( true );
		grabLoop = Glib::Thread::create( sigc::mem_fun(*this, &RL_ViewGUI::GrabFrames), true );
	}

	playing = true;
	stopping = false;
	pausing = false;

	playButton->set_sensitive(!playing);
	stopButton->set_sensitive(!stopping);
	pauseButton->set_sensitive(!pausing);
	folderChooser->set_sensitive(false);
}

/*! \brief OnPause: a callback function for the Pause button	
*/
void RL_ViewGUI::OnPause() {
	Glib::Mutex::Lock saveLock(playMutex);
	playing = false;
	stopping = false;
	pausing = true;

	playButton->set_sensitive(!playing);
	stopButton->set_sensitive(stopping);
	pauseButton->set_sensitive(!pausing);

}

/*! \brief OnStop: a callback function for the Stop button	
*/
void RL_ViewGUI::OnStop() {
	Glib::Mutex::Lock saveLock(playMutex);
	playing = false;
	stopping = true;
	pausing = false;

	playButton->set_sensitive(!playing);
	stopButton->set_sensitive(!stopping);
	pauseButton->set_sensitive(pausing);
	folderChooser->set_sensitive(true);

	SetLogo(rectified[0], logobuf);
	SetLogo(rectified[1], logobuf);
	SetLogo(stereoFrame, logobuf);
	SetLogo(imageFrame[0], logobuf);
	SetLogo(imageFrame[1], logobuf);

	fileOpened = false;
}

/*! \brief GrabFrames: thread routine for grabbing the frames from the sequence file
*/
void RL_ViewGUI::GrabFrames() {
	IplImage* iplTemp[2];
	int frameWidth = inputStream->GetFrameWidth();
	int frameHeight = inputStream->GetFrameHeight();
	int nChannels = inputStream->GetNumberOfChannels();
	int imageSize = frameWidth*frameHeight*nChannels;

	IplImage* view_color = cvCreateImage(cvSize(frameWidth, frameHeight), IPL_DEPTH_8U, 3);
	IplImage* view_gray = cvCreateImage(cvSize(frameWidth, frameHeight), IPL_DEPTH_8U, 1 );
	int selectedFrames[2];
	int lastSelectedFrames[] = {-1, -1};

	bool stereoReady;

	UpdateStereoParam();

	while(GetThreadStatus()) {
		Glib::Mutex::Lock playLock(playMutex);
		// get frame selection
		selectedFrames[0] = GetFrameIndex(0);
		selectedFrames[1] = GetFrameIndex(1);

		if(stopping) {
			playLock.release();
			filenumber = 0;
			stopEvent.emit();
			break;
		}

		if(playing) {
			bool read = inputStream->ReadFrame();
			if(!read) {
				// check if the next file is available
				filenumber++;
				read = OpenFile(filenumber);
			}

			// stop grabbing the frames if there is no more files available
			if(!read) {
				playLock.release();
				filenumber = 0;
				stopEvent.emit();
				break;
			}

			// set logos if no frames are selected for display
			if(selectedFrames[0] < 0)
				SetLogo(imageFrame[0], logobuf);


			if(selectedFrames[1] < 0)
				SetLogo(imageFrame[1], logobuf);

		}

		playLock.release();

		// display selected frames 
		if(selectedFrames[0] >= 0) {
			iplTemp[0] = inputStream->GrabFrame(selectedFrames[0]);
			if(nChannels == 1) {
				cvCvtColor(inputStream->GrabFrame(selectedFrames[0]), view_color, CV_GRAY2RGB);
				imageFrame[0]->UpdateBuffer((unsigned char*)view_color->imageData);
			}
			else {
				imageFrame[0]->UpdateBuffer((unsigned char*)iplTemp[0]->imageData);
			}

		}

		if(selectedFrames[1] >= 0) {
			iplTemp[1] = inputStream->GrabFrame(selectedFrames[1]);
			if(nChannels == 1) {
				cvCvtColor(iplTemp[1], view_color, CV_GRAY2RGB);
				imageFrame[1]->UpdateBuffer((unsigned char*)view_color->imageData);
			}
			else {
				imageFrame[1]->UpdateBuffer((unsigned char*)iplTemp[1]->imageData);
			}
		}

		// Display disparity map
		Glib::Mutex::Lock stereoLock(stereoMutex);
		if(selectedFrames[0] >= 0 && selectedFrames[1] >= 0) {
			if(lastSelectedFrames[0] != selectedFrames[0] || lastSelectedFrames[1] != selectedFrames[1]) {
				stereoReady = stereoProcessor->ReadCameraParamsFromFile(serials[selectedFrames[0]],serials[selectedFrames[1]], folderPath.c_str());
				lastSelectedFrames[0] = selectedFrames[0];
				lastSelectedFrames[1] = selectedFrames[1];
			}
		}
		else
			stereoReady = false;

		if(stereoReady)	{
			if(nChannels == 1) {
				stereoProcessor->RectifyLeft(iplTemp[0]);
				stereoProcessor->RectifyLeft(iplTemp[1]);
			}
			else {
				cvCvtColor( iplTemp[0], view_gray, CV_RGB2GRAY );
				stereoProcessor->RectifyLeft(view_gray);

				cvCvtColor( iplTemp[1], view_gray, CV_RGB2GRAY );
				stereoProcessor->RectifyRight(view_gray);
			}
			cvCvtColor( stereoProcessor->rectLeftImage, view_color, CV_GRAY2BGR );
			rectified[0]->UpdateBuffer((unsigned char*)view_color->imageData);

			cvCvtColor( stereoProcessor->rectRightImage, view_color, CV_GRAY2BGR );
			rectified[1]->UpdateBuffer((unsigned char*)view_color->imageData);

			stereoProcessor->ComputeDepthMap(stereoProcessor->rectLeftImage, stereoProcessor->rectRightImage,true);
			stereoProcessor->ConvertToColorDepth(view_color);
			stereoFrame->UpdateBuffer((unsigned char*)view_color->imageData);

		}
		else {
			SetLogo(rectified[0], logobuf);
			SetLogo(rectified[1], logobuf);
			SetLogo(stereoFrame, logobuf);
		}
		stereoLock.release();

		if(stereoReady) UpdateStereoParam();
	}

	cvReleaseImage( &view_color );
	cvReleaseImage( &view_gray );
}

/*! \brief OnDestroy: a callback function on delete signal
*/
bool RL_ViewGUI::OnDestroy(GdkEventAny*) {
	if(grabLoop != NULL) {
		SetThreadStatus(false);
		grabLoop->join();
		grabLoop = NULL;
	}
	Gtk::Main::quit();
	return true;
}


